#!/bin/bash

dirname=$(dirname $0)

. ${dirname}/fhgfs_setup.defaults


BASE_REPO_PATH=${base_repo_url}/dists
RPM_GPG_KEY="RPM-GPG-KEY-fhgfs"
RPM_GPG_KEY_URL="${base_repo_url}/gpg/${RPM_GPG_KEY}"
DEB_GPG_KEY="DEB-GPG-KEY-fhgfs"
DEB_GPG_KEY_URL="${base_repo_url}/gpg/${DEB_GPG_KEY}"

function add_repo_zypper()
{
  if [ "$1" = "" ]; then return; fi 
  if [ "$2" = "" ]; then return; fi 
  HOST=$1
  REPO=${BASE_REPO_PATH}/$2
  TMP_FILENAME=/tmp/repo.fhgfs.${HOST}.$(date +%s)
  cat <<EOF > ${TMP_FILENAME}
[FhGFS]
name=FhGFS
enabled=1
autorefresh=1
baseurl=${REPO}
path=/
type=yum
EOF
  eval scp $scpParameters ${TMP_FILENAME} ${HOST}:/etc/zypp/repos.d/fhgfs.repo
  rm ${TMP_FILENAME}
  cmd="cd /tmp; wget $wgetParameters ${RPM_GPG_KEY_URL}; rpm --import ${RPM_GPG_KEY}"
  eval ssh $sshParameter $HOST \"$cmd\"
}

function add_repo_yum()
{
  if [ "$1" = "" ]; then return; fi 
  if [ "$2" = "" ]; then return; fi 
  HOST=$1
  REPO=${BASE_REPO_PATH}/$2
  TMP_FILENAME=/tmp/repo.fhgfs.${HOST}.$(date +%s)
  cat <<EOF > ${TMP_FILENAME}
[FhGFS]
name=FhGFS
baseurl=${REPO}
enabled=1
gpgcheck=0

EOF
  eval scp $scpParameters ${TMP_FILENAME} ${HOST}:/etc/yum.repos.d/fhgfs.repo
  rm ${TMP_FILENAME} 
  cmd="cd /tmp; wget $wgetParameters ${RPM_GPG_KEY_URL}; rpm --import ${RPM_GPG_KEY}"
  eval ssh $sshParameter $HOST \"$cmd\"
}

function add_repo_apt()
{
  if [ "$1" = "" ]; then return; fi
  if [ "$2" = "" ]; then return; fi
  HOST=$1
#  REPO=${BASE_REPO_PATH}/$2
  TMP_FILENAME=/tmp/repo.fhgfs.${HOST}.$(date +%s)

  if [ "$2" = "debian7" ]
  then 
    REPO="deb7"
  elif [ "$2" = "debian6" ]
  then 
    REPO="deb6"
  else
    REPO="deb5"
  fi 
 
  cmd="echo \"deb ${base_repo_url} ${REPO} non-free\" > /etc/apt/sources.list.d/fhgfs.list"
  eval ssh $sshParameter $HOST \"$cmd\"
  cmd="wget $wgetParameters ${DEB_GPG_KEY_URL} -O- | apt-key add -"
  eval ssh $sshParameter $HOST \"$cmd\"
  cmd="apt-get update"
  eval ssh $sshParameter $HOST \"$cmd\"
}



function check_package_manager()
{
 HOST=$1
 # yum = 1
 cmd="which yum"
 eval ssh $sshParameter ${HOST} \"$cmd\" > /dev/null 2>&1
 if [ $? -eq 0 ]
 then 
  return 1
 fi

 # zypper = 2
 cmd="which zypper"
 eval ssh $sshParameter ${HOST} \"$cmd\" > /dev/null 2>&1
 if [ $? -eq 0 ]
 then 
  return 2
 fi

 # unknown = -1
 return -1
}

if [ "$1" = "" ]; then exit 1; else HOST=$1; fi
if [ "$2" = "" ]; then exit 1; else DISTRI=$2; fi

# determine package manager
# if the distri is debian then it's apt otherwise check for zypper or yum
echo ${DISTRI} | grep "debian" > /dev/null 2>&1
if [ $? -eq 0 ]
then
 # create apt repo 
  add_repo_apt ${HOST} ${DISTRI}
else
 check_package_manager ${HOST}
 retval=$?
 if [ $retval -eq 1 ]
 then
   add_repo_yum ${HOST} ${DISTRI}
 elif [ $retval -eq 2 ]
 then
   add_repo_zypper ${HOST} ${DISTRI}
 else
   if [ "$3" != "" ]
   then
     echo ${HOST} >> $3
   fi 
   exit 1
 fi
fi
sleep 1

