#!/bin/bash

dirname=$(dirname $0)

. ${dirname}/fhgfs_setup.defaults


check_suse_client_dependencies()
{
   if [ "$1" = "" ]
   then
      return 1
   fi

   host=$1

   if [ "$2" = "" ]
   then
      outFile=${logFile}
   else   
      outFile=$2
   fi
 
   #SuSe with package manager zypper
   cmd="which zypper"
   eval ssh $sshParameter ${host} \"$cmd\" > /dev/null
   if [ $? -eq 0 ]
   then
      cmd="zypper search -i kernel-source | grep kernel-source"
      eval ssh $sshParameter ${host} \"${cmd}\" > /dev/null 2> ${outFile}

      if [ $? -ne 0 ]
      then
         flock ${tmpClientWarning} /bin/bash -c "echo ${host} >> ${tmpClientWarning}" 2> ${outFile}
         echo "WARNING: ${host} - Please check if the kernel module development packages (kernel-source) is installed for the current kernel version." >> ${outFile} 2>&1
         return 1
      fi

   #SuSe with package manager yum
   else
      cmd="yum list installed kernel-source | grep kernel-source"
      eval ssh $sshParameter ${host} \"${cmd}\" > /dev/null 2> ${outFile}     

      if [ $? -ne 0 ]
      then
         flock ${tmpClientWarning} /bin/bash -c "echo ${host} >> ${tmpClientWarning}" 2> ${outFile}
         echo "WARNING: ${host} - Please check if the kernel module development packages (kernel-source) is installed for the current kernel version." >> ${outFile} 2>&1
	 return 1
      fi
   fi

   return 0
}

check_redhat_client_dependencies()
{
   if [ "$1" = "" ]
   then
      return 1
   fi

   host=$1

   if [ "$2" = "" ]
   then
      outFile=${logFile}
   else   
      outFile=$2
   fi

   #RedHat with package manager yum
   cmd="yum list installed kernel-devel | grep kernel-devel"
   eval ssh $sshParameter ${host} \"${cmd}\" > /dev/null 2> ${outFile}

   if [ $? -ne 0 ]
   then
      flock ${tmpClientWarning} /bin/bash -c "echo ${host} >> ${tmpClientWarning}" 2> ${outFile}
      echo "WARNING: ${host} - Please check if the kernel module development package (kernel-devel) is installed for the current kernel version." >> ${outFile} 2>&1
      return 1
   fi

   return 0
}

check_debian_client_dependencies()
{
   if [ "$1" = "" ]
   then
      return 1
   fi

   host=$1

   if [ "$2" = "" ]
   then
      outFile=${logFile}
   else   
      outFile=$2
   fi

   #Debian with package manager dpkg
   cmd="dpkg -l linux-headers* | grep linux-headers | grep ii |grep meta"
   eval ssh $sshParameter ${host} \"${cmd}\" > /dev/null 2> ${outFile}

   if [ $? -ne 0 ]
   then
      flock ${tmpClientWarning} /bin/bash -c "echo ${host} >> ${tmpClientWarning}" 2> ${outFile}
      echo "WARNING: ${host} - Please check if the kernel module development package (linux-headers) is installed for the current kernel version." >> ${outFile} 2>&1
      return 1
   fi

   return 0
}

install_package()
{
   if [ "$1" = "" ]
   then
      return 1
   else
      hostline=$1
   fi

   if [ "$2" = "" ]
   then
      return 1
   else
      description=$2
   fi

   if [ "$3" = "" ]
   then
      return 1
   else
      package=$3
   fi

   if [ "$4" = "" ]
   then
      outFile=${logFile}
   else   
      outFile=$4
   fi

   if [ "$5" = "" ]
   then
      failedFile="/dev/null"
   else
      failedFile=$5
   fi

   echo "" >> ${outFile}
   echo "--------------------" >> ${outFile}
   date >> ${outFile}
   echo "" >> ${outFile}
   
   host=$(echo ${hostline} | cut -f 1 -d ",")
   arch=$(echo ${hostline} | cut -f 2 -d ",")
   dist=$(echo ${hostline} | cut -f 4 -d ",")
   
   #this is necessary because older yum version install both 
   # 64 bit as well as 32 bit on 64bit machines if you do not force 64bit only
   if [ "${arch}" = "x86_64" ]
   then
      YUM_EXT=".x86_64"
   fi

   echo "Installing ${description} on host ${host}" >> ${outFile}
   echo "--------------------" >> ${outFile}
  
   echo $dist | grep debian > /dev/null 2>&1 
   if [ $? -eq 0 ]
   then
      if [ "${package}" = "fhgfs-client" ]
      then
         check_debian_client_dependencies ${host} ${outFile}

         # No architecture extension for the package is required, because the 
         # client is a noarch package
         YUM_EXT=""
      fi
      cmd="apt-get -y --allow-unauthenticated"
      YUM_EXT=""
   else
      echo $dist | grep suse > /dev/null 2>&1 
      if [ $? -eq 0 ]
      then
         if [ "${package}" = "fhgfs-client" ]
         then
            check_suse_client_dependencies ${host} ${outFile}

            # No architecture extension for the package is required, because the 
            # client is a noarch package
            YUM_EXT=""
         fi
      else
         if [ "${package}" = "fhgfs-client" ]
         then
            check_redhat_client_dependencies ${host} ${outFile}

            # No architecture extension for the package is required, because the 
            # client is a noarch package
            YUM_EXT=""
         fi  
      fi
      cmd="yum -y"
      # if we have a zypper repository take zypper
      cmd_check="which zypper"
      eval ssh $sshParameter ${host} \"$cmd_check\" > /dev/null
      if [ $? -eq 0 ];
         then
            cmd="zypper -n --no-gpg-checks"
            YUM_EXT=""
         fi
   fi

   cmd_remote="${cmd} install ${package}${YUM_EXT}"
   eval ssh $sshParameter ${host} \"$cmd_remote\" >> ${outFile} 2>&1

   if [ $? -ne 0 ]
   then
      error="Failed to install ${description} on host ${host}"
      echo $error >> ${outFile}
      echo ${host} >> ${failedFile}
      return 1
   fi

   return 0
}

deploy_meta()
{
   if [ "$1" = "" ]
   then
      return 1   
   else
      host=$1
   fi

   if [ "$2" = "" ]
   then
      outFile=${logFile}
   else
      outFile=$2
   fi

   if [ "$3" = "" ]
   then
      failedFile="/dev/null"
   else
      failedFile=$3
   fi

   hostline=$(grep "$1" $metaFile)
   install_package "$hostline" "FhGFS Meta Server" fhgfs-meta $outFile $failedFile
   if [ $? -eq 0 ]
   then 
     bash /opt/fhgfs/setup/fhgfs_setup.write_config.sh ${host} /etc/fhgfs/fhgfs-meta.conf
   else
     return 1
   fi
}

deploy_storage()
{
   if [ "$1" = "" ]
   then
      return 1
   fi

   if [ "$2" = "" ]
   then
      outFile=${logFile}
   else
      outFile=$2
   fi

   if [ "$3" = "" ]
   then
      failedFile="/dev/null"
   else
      failedFile=$3
   fi
   
   hostline=$(grep "$1" $storageFile)
   install_package "$hostline" "FhGFS Storage Server" fhgfs-storage $outFile $failedFile
   if [ $? -eq 0 ]
   then 
     bash /opt/fhgfs/setup/fhgfs_setup.write_config.sh ${host} /etc/fhgfs/fhgfs-storage.conf
   else
     return 1
   fi
}

deploy_mgmtd()
{
   if [ "$1" = "" ]
   then
      return 1
   fi

   if [ "$2" = "" ]
   then
      outFile=${logFile}
   else
      outFile=$2
   fi

   if [ "$3" = "" ]
   then
      failedFile="/dev/null"
   else
      failedFile=$3
   fi

   hostline=$(grep "$1" $managementFile)
   install_package "$hostline" "FhGFS Management Daemon" fhgfs-mgmtd $outFile $failedFile
   if [ $? -eq 0 ]
   then 
     bash /opt/fhgfs/setup/fhgfs_setup.write_config.sh ${host} /etc/fhgfs/fhgfs-mgmtd.conf
   else
     return 1
   fi
}


deploy_opentk()
{
   if [ "$1" = "" ]
   then
      return 1
   else
      HOST=$1
   fi

   if [ "$2" = "" ]
   then
      outFile=${logFile}
   else
      outFile=$2
   fi

   if [ "$3" = "" ]
   then
      failedFile="/dev/null"
   else
      failedFile=$3
   fi

   hostline=$(cat ${metaFile} ${storageFile} ${clientFile} ${managementFile} | sort | uniq | grep "${HOST}")
   install_package "$hostline" "FhGFS Opentk library" fhgfs-opentk-lib $outFile $failedFile
   if [ $? -ne 0 ]; then return 1; fi

   IBLINE=$(grep "${HOST}" ${ib_hosts_file})
#   if [ "${IBLINE}" != "" ]
#   then
#     echo "Infiniband support for this host was disabled by user. Leaving opentk library without support for Infiniband." >> ${outFile}
#   else
# user requested IB support
   IB_OPTIONS=$(echo ${IBLINE} | cut -f 2 -d ",")
   IFS=" "
   COMPILE_OPTIONS=""
   for PARAM in ${IB_OPTIONS}
   do
     eval $PARAM
   done
   if [ "${OFED_INCLUDE_PATH}" = "" ] && [ "${OFED_LIB_PATH}" = "" ]
   then
     eval ssh $sshParameter ${HOST} ${OPENTK_IB_SCRIPT} >> ${outFile}
     if [ $? -eq 0 ]
     then 
       return 1
     fi    
  else
   # compile options are given... compile
     FHGFS_OPENTK_IBVERBS=1
     export FHGFS_OPENTK_IBVERBS
     export OFED_INCLUDE_PATH
     export OFED_LIB_PATH
     echo "Compiling opentk library with the following options : " >> ${outFile}
     echo "FHGFS_OPENTK_IBVERBS=${FHGFS_OPENTK_IBVERBS} OFED_INCLUDE_PATH=${OFED_INCLUDE_PATH} OFED_LIB_PATH=${OFED_LIB_PATH}" >> ${outFile}
     echo "" >> ${outFile}
     
TOOLS_TO_CHECK="make g++"
     for tool in $TOOLS_TO_CHECK
     do
       cmd="which ${tool}"
       eval ssh $sshParameter ${HOST} \"$cmd\" > /dev/null 2>&1
       if [ $? -ne 0 ]
       then
         echo "" >> ${outFile}
         error="Failed to compile opentk library on host ${HOST}. No '${tool}' found."
         echo $error >> $outFile
         echo ${HOST} >> ${failedFile}
         return 1
       fi
     done
     
     cmd="${OPENTK_IB_SCRIPT} -r"
     eval ssh $sshParameter ${HOST} \"$cmd\" > /dev/null 2>> ${outFile}
     if [ $? -eq 0 ]
     then
       echo "Successfully compiled opentk library." >> ${outFile}
     else
       echo "Failed to compile opentk library... trying to use standard library." >> ${outFile}
       eval ssh $sshParameter ${HOST} ${OPENTK_IB_SCRIPT} >> ${outFile}
       return 1
     fi
   fi

   return 0
}

deploy_client()
{
   if [ "$1" = "" ]
   then
      return 1
   fi

   if [ "$2" = "" ]
   then
      outFile=${logFile}
   else
      outFile=$2
   fi

   if [ "$3" = "" ]
   then
      failedFile="/dev/null"
   else
      failedFile=$3
   fi

   hostline=$(grep "$1" $clientFile)

   deploy_utils $1 $2 $3

   install_package "$hostline" "FhGFS Helper Daemon" fhgfs-helperd $outFile $failedFile  
   if [ $? -eq 0 ]
   then 
     bash /opt/fhgfs/setup/fhgfs_setup.write_config.sh ${host} /etc/fhgfs/fhgfs-helperd.conf
   else
     return 1
   fi
   
   #this package will be installed with the package dependencies and not with the script
   #install_package "$hostline" "FhGFS Client Opentk module" fhgfs-client-opentk-src $outFile $failedFile
   install_package "$hostline" "FhGFS Client module" fhgfs-client $outFile $failedFile
   if [ $? -ne 0 ]; then return 1; fi   

   # check for ib options and compile client
   IB_OPTIONS=$(grep ${host} ${ib_hosts_file} | cut -f 2 -d ",")
   if [ "${IB_OPTIONS}" = "" ]
   then
     # no IB
     COMPILE_OPTIONS="-j8"
   else 
     for PARAM in ${IB_OPTIONS}
     do
       eval $PARAM
     done
     if [ "${OFED_KERNEL_INCLUDE_PATH}" != "" ]
     then
       COMPILE_OPTIONS="-j8 FHGFS_OPENTK_IBVERBS=1 OFED_INCLUDE_PATH=${OFED_KERNEL_INCLUDE_PATH}"
     else
       COMPILE_OPTIONS="-j8 FHGFS_OPENTK_IBVERBS=1"
     fi
   fi
   # look if make is installed 
   TOOLS_TO_CHECK="make g++"
   for tool in $TOOLS_TO_CHECK
   do
     cmd="which ${tool}"
     eval ssh $sshParameter ${host} \"$cmd\" > /dev/null 2>&1
     if [ $? -ne 0 ]
     then
       echo "" >> ${outFile}
       error="Failed to compile client module on host ${host}. No '${tool}' found."
       echo $error >> $outFile
       echo ${host} >> ${failedFile}
       return 1
     fi
   done
   cmd="sed -i 's!buildArgs=.*!buildArgs=${COMPILE_OPTIONS}!g' ${AUTOBUILD_CONFIG}"
   eval ssh $sshParameter ${host} \"$cmd\" >> ${outFile} 2>&1
   cmd="/etc/init.d/fhgfs-client rebuild"
   eval ssh $sshParameter ${host} \"$cmd\" >> ${outFile} 2>&1
   cmd="rm -f /var/lib/fhgfs/client/force-auto-build"
   eval ssh $sshParameter ${host} \"$cmd\" >> ${outFile} 2>&1
   if [ $? -ne 0 ]
   then
      echo "" >> ${outFile}
      error="Failed to compile client module on host ${host}"
      echo $error >> $outFile
      echo "Options have been : ${COMPILE_OPTIONS}" >> $outFile
      echo ${host} >> ${failedFile}
      return 1
   fi

   bash /opt/fhgfs/setup/fhgfs_setup.write_config.sh ${host} /etc/fhgfs/fhgfs-client.conf
   bash /opt/fhgfs/setup/fhgfs_setup.write_mounts.sh ${host}
}

deploy_utils()
{
   if [ "$1" = "" ]
   then
      return 1
   fi

   if [ "$2" = "" ]
   then
      outFile=${logFile}
   else
      outFile=$2
   fi

   if [ "$3" = "" ]
   then
      failedFile="/dev/null"
   else
      failedFile=$3
   fi

   hostline=$(grep "$1" $clientFile)
   install_package "$hostline" "FhGFS Utilities" fhgfs-utils $outFile $failedFile
   if [ $? -ne 0 ]
   then
     return 1
   fi
}

deploy_common()
{
   if [ "$1" = "" ]
   then
      return 1
   else
      HOST=$1
   fi

   if [ "$2" = "" ]
   then
      outFile=${logFile}
   else
      outFile=$2
   fi

   if [ "$3" = "" ]
   then
      failedFile="/dev/null"
   else
      failedFile=$3
   fi

   hostline=$(cat ${metaFile} ${storageFile} ${clientFile} ${managementFile} | sort | uniq | grep "${HOST}")
   install_package "$hostline" "FhGFS Common" fhgfs-common $outFile $failedFile
   if [ $? -ne 0 ]
   then 
     return 1
   fi
}


if [ "$1" = "" ];then exit 1; fi
if [ "$2" = "" ];then exit 1; fi

if [ "$1" = "opentk" ]
then
   #this package will be installed with the package dependencies and not with the script
   #deploy_common $2 $3 $4
   deploy_opentk $2 $3 $4
   exit $?
elif [ "$1" = "meta" ]
then
   deploy_meta $2 $3 $4
   exit $?
elif [ "$1" = "storage" ]
then
   deploy_storage $2 $3 $4
   exit $?
elif [ "$1" = "client" ]
then
   deploy_client $2 $3 $4
   exit $?
elif [ "$1" = "mgmtd" ]
then
   deploy_mgmtd $2 $3 $4
   exit $?
elif [ "$1" = "utils" ]
then
   deploy_utils $2 $3 $4
   exit $?
else
   exit 1
fi
